% This file is part of "Motion Correction Microscopy" published as: 
% 
% Kumar et. al., "A Motion Correction Framework for Time Series 
% Sequences in Microscopy Images." Microscopy and Microanalysis. 2012. 
% 
% "Motion Correction Microscopy" is free software: you can redistribute it and/or modify
% it under the terms of the GNU Lesser Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% "Motion Correction Microscopy" is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Lesser Public License for more details.
% 
% You should have received a copy of the GNU Lesser Public License
% along with "Motion Correction Microscopy".  If not, see <http://www.gnu.org/licenses/>.
% 
% %%%%%%%%%%%%%%%%%%%%%%%% COPYRIGHT %%%%%%%%%%%%%%%%%%%%%%%%
% 
% Copyright (c) 2012, Kumar, Ankur N (ankur.kumar@vanderbilt.edu)
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without 
% modification, are permitted provided that the following conditions are 
% met:
% 
%     * Redistributions of source code must retain the above copyright 
%       notice, this list of conditions and the following disclaimer.
%     * Redistributions in binary form must reproduce the above copyright 
%       notice, this list of conditions and the following disclaimer in 
%       the documentation and/or other materials provided with the distribution
%     * Cite the Reference: 
%         
%         * Kumar et. al., "A Motion Correction Framework for Time Series 
%         Sequences in Microscopy Images." Microscopy and Microanalysis. 2012. 
%       
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
% AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
% IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
% ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
% LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
% CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
% SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
% CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
% ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
% POSSIBILITY OF SUCH DAMAGE.
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Please cite the publication:
% 
% Kumar et. al., "A Motion Correction Framework for Time Series 
% Sequences in Microscopy Images." Microscopy and Microanalysis. 2012. 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%compute 2nd order derivatives, at different scales (sigvec, 1xn array)
%delta, is the size of the gaussian kernel, (delta x delta)
%gamma is normalization for vesselness
function [hessianDetStk, hessianEigStk] = computeFeatureValues(stk_vessl, stk_rbc, sigvec, delta, gamma, beta, minpkdist,thresh)

%FIX CODE to account for stk_vessl, stk_rbc as different stks
%Lxx,etc should be ch1(vessel) ch2(rbc)
%computeHessianDet should only process ch2 and store DoH from ch2
%computeHessianEig should only process ch1 and store vesselness from ch1

%initialize all hessian datastructures
depth = length(stk_vessl); %numslices
width = stk_vessl(1,1).width; %col
height = stk_vessl(1,1).height; %row
sigma = 0;
hessianDetStk = [];
hessianEigStk = [];
Lxx = struct('data',{zeros(height,width), zeros(height,width)});
Lyy = struct('data',{zeros(height,width), zeros(height,width)});
Lxy = struct('data',{zeros(height,width), zeros(height,width)});

% Derivatives stack, 2nd order
derStk = repmat(struct('Lxx',Lxx,'Lyy',Lyy,'Lxy',Lxy),[1,depth]);
% scaledDerStk = repmat(struct('sigma',sigma,'derStk',derStk),[1,length(sigvec)]);

%initialize Gaussian derivative kernels
Gxx_kernel = zeros(delta, delta);
Gyy_kernel = zeros(delta, delta);
Gxy_kernel = zeros(delta, delta);
offset = floor(delta/2)+1;

for sig_index=1:length(sigvec)
    sigma = sigvec(sig_index);
    
    %scaledDerStk(sig_index).sigma = sigma;
    %construct Gaussian derivative kernels
    A = 1/(2*pi*sigma^2);
    Gxx_func = @(x, y)((x^2 - sigma^2)/(sigma^4))*A*exp(-(x^2+y^2)/(2*sigma^2));
    Gyy_func = @(x, y)((y^2 - sigma^2)/(sigma^4))*A*exp(-(x^2+y^2)/(2*sigma^2));
    Gxy_func = @(x, y)(x*y/sigma^4)*A*exp(-(x^2+y^2)/(2*sigma^2));
    
    for i=-floor(delta/2):floor(delta/2) %row, y-dimension in an image
        for j=-floor(delta/2):floor(delta/2) %col, x-dimension in an image
            Gxx_kernel(i+offset,j+offset) = Gxx_func(j, i);
            Gyy_kernel(i+offset,j+offset) = Gyy_func(j, i);
            Gxy_kernel(i+offset,j+offset) = Gxy_func(j, i);
        end
    end
    
%     Gxx_kernel
%     Gyy_kernel
%     Gxy_kernel
%     figure(sig_index)
%     subplot(131)
%     image(Gxx_kernel.*1000)
%     subplot(132)
%     image(Gyy_kernel.*1000)
%     subplot(133)
%     image(Gxy_kernel.*1000)
    %build Hessians data structures for sigma
    tic
    for k=1:depth
        %populate hessStk(1:depth).3Hessians(ch1 or ch2).data
        ch1_data = double(stk_vessl(k).data); %vessels, use in computeHessianEig
        ch2_data = double(stk_rbc(k).data); %rbc, use in computeHessianDet
        derStk(k).Lxx(1,1).data = conv2(ch1_data, Gxx_kernel, 'same'); %channel 1
        derStk(k).Lxx(1,2).data = conv2(ch2_data, Gxx_kernel, 'same'); %channel 2
        derStk(k).Lyy(1,1).data = conv2(ch1_data, Gyy_kernel, 'same'); %channel 1
        derStk(k).Lyy(1,2).data = conv2(ch2_data, Gyy_kernel, 'same'); %channel 2
        derStk(k).Lxy(1,1).data = conv2(ch1_data, Gxy_kernel, 'same'); %channel 1
        derStk(k).Lxy(1,2).data = conv2(ch2_data, Gxy_kernel, 'same'); %channel 2
        fprintf('Computed derivatives, scale=%d, Completed...(%d/%d)\n',sigma,k,depth);
        
%         figure(1)
%         imshow(uint8(255*(abs(derStk(k).Lxx(1,1).data./max(derStk(k).Lxx(1,1).data(:))))) );
%         figure(2)
%         imshow(uint8(255*(abs(derStk(k).Lyy(1,1).data./max(derStk(k).Lyy(1,1).data(:)))) ));
%         figure(3)
%         imshow(uint8(255*(derStk(k).Lxy(1,1).data./max(derStk(k).Lxy(1,1).data(:)))) );
%         figure(4)
%         imshow(uint8(255*(abs(derStk(k).Lxx(1,2).data./max(derStk(k).Lxx(1,2).data(:)))) ));
%         figure(5)
%         imshow(uint8(255*(abs(derStk(k).Lyy(1,2).data./max(derStk(k).Lyy(1,2).data(:)))) ));
%         figure(6)
%         imshow(uint8(255*(derStk(k).Lxy(1,2).data./max(derStk(k).Lxy(1,2).data(:)))) );
%         
%         
%         
% %         
% %         
% %         Gxx_kernel
% %         Gyy_kernel        
% %         Gxy_kernel
% %         
%         pause
        
    end
    fprintf('Computed derivatives, %dth scale, Completed...\n',sig_index);
    toc
    %compute hessian determinants at different scales, using sig_vec
    tic
    %%%%%UNCOMMENT THE FOLLOWING
   hessianDetStk.data{sig_index} = computeHessianDet(derStk, sigma,minpkdist,thresh);
   hessianDetStk.sigma{sig_index} = sigma;
    toc
    fprintf('Computed hessian determinant, %dth scale, Completed...\n',sig_index);
    %compute hessian eigenvalues at different scales, using sig_vec
    tic
    %need to iterate through all sig_index, to find MAX Vesselness
   % hessianEigStk.data{sig_index} = computeHessianEig(derStk, sigma, gamma, beta, -1);
  %  hessianEigStk.sigma{sig_index} = sigma;
    toc
    
    fprintf('Computed hessian Eigenvalues, %dth scale, Completed...\n',sig_index);
    %scaledDerStk(sig_index).derStk = derStk;
    
end