% This file is part of "Motion Correction Microscopy" published as: 
% 
% Kumar et. al., "A Motion Correction Framework for Time Series 
% Sequences in Microscopy Images." Microscopy and Microanalysis. 2012. 
% 
% "Motion Correction Microscopy" is free software: you can redistribute it and/or modify
% it under the terms of the GNU Lesser Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% "Motion Correction Microscopy" is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Lesser Public License for more details.
% 
% You should have received a copy of the GNU Lesser Public License
% along with "Motion Correction Microscopy".  If not, see <http://www.gnu.org/licenses/>.
% 
% %%%%%%%%%%%%%%%%%%%%%%%% COPYRIGHT %%%%%%%%%%%%%%%%%%%%%%%%
% 
% Copyright (c) 2012, Kumar, Ankur N (ankur.kumar@vanderbilt.edu)
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without 
% modification, are permitted provided that the following conditions are 
% met:
% 
%     * Redistributions of source code must retain the above copyright 
%       notice, this list of conditions and the following disclaimer.
%     * Redistributions in binary form must reproduce the above copyright 
%       notice, this list of conditions and the following disclaimer in 
%       the documentation and/or other materials provided with the distribution
%     * Cite the Reference: 
%         
%         * Kumar et. al., "A Motion Correction Framework for Time Series 
%         Sequences in Microscopy Images." Microscopy and Microanalysis. 2012. 
%       
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
% AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
% IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
% ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
% LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
% CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
% SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
% CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
% ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
% POSSIBILITY OF SUCH DAMAGE.
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Please cite the publication:
% 
% Kumar et. al., "A Motion Correction Framework for Time Series 
% Sequences in Microscopy Images." Microscopy and Microanalysis. 2012. 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%compute Eigenvalues, Vesselness of Hessian given Derivative Stk and sigma
%multiply by scale=sigma^gamma, equation(2) in Frangi et al.
%eigdata = {lambda1}, {lambda2} where |lambda1|<=|lambda2|
%Beta and c control sensitivity of line filter
%if c<0, 
%
function eigHessStk = computeHessianEig(derStk, sigma, beta, c)
cflag = (c>0);

depth = length(derStk);
sz = size(derStk(1).Lxx(1,1).data); %sz(1) is rows(height), sz(2) is cols(width)
totlen = sz(1)*sz(2);

%lambda1 lamdba2
eigHess = struct('lambdas',{zeros(sz), zeros(sz)});
eigVec = struct('vector',{zeros(sz), zeros(sz)});

%channel1 channel2
eigHessStk = repmat(struct('eig',eigHess, 'eigVec1', eigVec, 'eigVec2', eigVec, 'normf',{zeros(sz)},  'vesselness',{zeros(sz)}),[1,depth]);
scale = 1;
for k=1:depth
    cmaxch1 = realmin;
    %construct Hessian Matrix
    %reshape Lxx Lyy Lxy to 1-D vector, Channel1 (vessels)
    Lxx_vec = scale.*reshape(derStk(k).Lxx(1,1).data,1,[]);
    Lyy_vec = scale.*reshape(derStk(k).Lyy(1,1).data,1,[]);
    Lxy_vec = scale.*reshape(derStk(k).Lxy(1,1).data,1,[]);
    lambdas = zeros(2,length(Lxx_vec));
    vec1 = zeros(2,length(Lxx_vec)); %1st element of eigvector1
    vec2 = zeros(2,length(Lxx_vec)); %1st element of eigvector2
    normf = zeros(1,length(Lxx_vec));
    for i=1:totlen
        h = [Lxx_vec(i), Lxy_vec(i); Lxy_vec(i), Lyy_vec(i)]; %hessian
        [utemp,ltemp] = eig(h); %lambda1, lambda2, but |lambda1|?|lambda2|
        
        ltemp = diag(ltemp);
        [~, inx] = sort(abs(ltemp));
        ltemp = ltemp(inx); %sorted, |lambda1|<|lambda2|
        lambdas(:,i) = ltemp; %eigenvalues, abs(lambda1<lambda2)
        vec1(:,i) = utemp(:,inx(1)); %store 1st eigenvector, also sorted
        vec2(:,i) = utemp(:,inx(2)); %store 2nd eigenvector, also sorted
        normf(i) = norm(h,'fro'); %frobenius norm, second order structureness
        if(normf(i)>cmaxch1) %find maximum frobenius norm
            cmaxch1 = normf(i);
        end
    end
    
    %reshape/store lambdas
    eigHessStk(k).eig(1,1).lambdas = reshape(lambdas(1,:), [sz(1) sz(2)]);
    eigHessStk(k).eig(1,2).lambdas = reshape(lambdas(2,:), [sz(1) sz(2)]);
    %store eigenvectors, each eigVec is (1,1) stores vec(1) and (1,2)
    %stores vec(2) element, eigvectors are sorted according to eigvalue
    eigHessStk(k).eigVec1(1,1).vector = reshape(vec1(1,:), [sz(1) sz(2)]);
    eigHessStk(k).eigVec1(1,2).vector = reshape(vec1(2,:), [sz(1) sz(2)]);
    eigHessStk(k).eigVec2(1,1).vector = reshape(vec2(1,:), [sz(1) sz(2)]);
    eigHessStk(k).eigVec2(1,2).vector = reshape(vec2(2,:), [sz(1) sz(2)]);
    
    eigHessStk(k).normf = reshape(normf, [sz(1) sz(2)]);
    cmaxch1 = 0.5*cmaxch1; %threshold c
    if(cflag) 
        cmaxch1=c; 
    end
    %compute Vesselness for scale/sigma
    v_0 = computeVesselness(lambdas(1,:), lambdas(2,:), normf, beta, cmaxch1);
    eigHessStk(k).vesselness = reshape(v_0, [sz(1) sz(2)]);
    eigHessStk(k).c_thresh = cmaxch1;

    
    fprintf('Computed eigenvalues/vesselness, scale=%d, Completed...(%d/%d)\n',sigma,k,depth);
end

