% This file is part of "Motion Correction Microscopy" published as: 
% 
% Kumar et. al., "A Motion Correction Framework for Time Series 
% Sequences in Microscopy Images." Microscopy and Microanalysis. 2012. 
% 
% "Motion Correction Microscopy" is free software: you can redistribute it and/or modify
% it under the terms of the GNU Lesser Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% "Motion Correction Microscopy" is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Lesser Public License for more details.
% 
% You should have received a copy of the GNU Lesser Public License
% along with "Motion Correction Microscopy".  If not, see <http://www.gnu.org/licenses/>.
% 
% %%%%%%%%%%%%%%%%%%%%%%%% COPYRIGHT %%%%%%%%%%%%%%%%%%%%%%%%
% 
% Copyright (c) 2012, Kumar, Ankur N (ankur.kumar@vanderbilt.edu)
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without 
% modification, are permitted provided that the following conditions are 
% met:
% 
%     * Redistributions of source code must retain the above copyright 
%       notice, this list of conditions and the following disclaimer.
%     * Redistributions in binary form must reproduce the above copyright 
%       notice, this list of conditions and the following disclaimer in 
%       the documentation and/or other materials provided with the distribution
%     * Cite the Reference: 
%         
%         * Kumar et. al., "A Motion Correction Framework for Time Series 
%         Sequences in Microscopy Images." Microscopy and Microanalysis. 2012. 
%       
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
% AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
% IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
% ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
% LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
% CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
% SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
% CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
% ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
% POSSIBILITY OF SUCH DAMAGE.
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Please cite the publication:
% 
% Kumar et. al., "A Motion Correction Framework for Time Series 
% Sequences in Microscopy Images." Microscopy and Microanalysis. 2012. 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Normalized Cross Correlation (NCC)
%Evaluate NCC against template and find out frame numbers with lots of movement
%May 23 2012, for manuscript revision
%Input: templateVesselStack, maxVesselStack
%REGISTRATION_FLAG: perform the registration, otherwise just evaluate NCC
%suffix_string: suffix for output filenames
%SC_FLAG: Use deformed vesselness after motion correction
%Output: registered maxVesselStack, if REGISTRATION_FLAG = 1
%Output: ncc_vec contains NCC values for all frames of maxVesslStk,  a file with NCC values for each frame is stored separately
%Output: ranges - logic values (TRUE, FALSE) for performing motion-correction

function [nccVesslStk, ncc_vec, ranges] = ncc_registration(filepath1, filename1, maxTEMPLATEVesslStk, maxVesslStk, REGISTRATION_FLAG, suffix_string, SC_FLAG)
nccVesslStk = [];
lz = length(maxVesslStk); %length of maxVesslStk
sz = size(maxVesslStk(1).vesselness);
template_vessl = maxTEMPLATEVesslStk.vesselness_template; %fetch template
c0 = normxcorr2(template_vessl, template_vessl);
[max_c, imax] = max(abs(c0(:)));
[ypeak, xpeak] = ind2sub(size(c0), imax(1));
corr_offset0 = [xpeak, ypeak];
rect_offset = [0, 0];
offset0 = corr_offset0 + rect_offset;
xoffset0 = offset0(1); yoffset0 = offset0(2);
ncc_vec = zeros(lz, 2);

for k=1:lz
    slice_vessl = maxVesslStk(k).vesselness;
    if(SC_FLAG)
        slice_vessl = maxVesslStk(k).deformvesselness;
    end
    c = normxcorr2(template_vessl, slice_vessl); %NCC
    %offset between template and slice depends on location of the peak in cross-correlation matrix
    [max_c, imax] = max(abs(c(:)));
    [ypeak, xpeak] = ind2sub(size(c), imax(1));
    corr_offset = [xpeak, ypeak];
    offset = corr_offset + rect_offset;
    xoffset = offset(1); yoffset = offset(2);
    dst = sqrt((xoffset-xoffset0).^2 + (yoffset-yoffset0).^2);
    ncc_vec(k,:) = [k, dst];
    
end

ranges = ncc_vec(:,2)>sqrt(2); %frames requiring motion correction
%output files
outfn1 = strcat(filepath1, filename1);
outfname1 = strcat(outfn1, '_sigMAX','_del15','_NCC_Deform_',suffix_string,'.tif');
outfnamecsv = strcat(outfn1, '_sigMAX','_del15','_NCC_',suffix_string,'.txt');
ncc_range_vec = [ncc_vec, ranges];
dlmwrite(outfnamecsv, ncc_range_vec); %write out NCC and ranges values
 disp(['Saved NCC file.. ' outfnamecsv]);
%perform Registration if set
rect_offset = -1*[size(template_vessl,2), size(template_vessl,1)];
if(REGISTRATION_FLAG)
    for k=1:lz
        slice_vessl = maxVesslStk(k).vesselness;
        if(SC_FLAG)
            slice_vessl = maxVesslStk(kx).deformvesselness;
        end
        c = normxcorr2(template_vessl, slice_vessl); %NCC
        %offset between template and slice depends on location of the peak in cross-correlation matrix
        [max_c, imax] = max(abs(c(:)));
        [ypeak, xpeak] = ind2sub(size(c), imax(1));
        corr_offset = [xpeak, ypeak];
        offset = corr_offset - corr_offset0;
        xoffset = offset(1); yoffset = offset(2);
        if(xoffset < 0) %for example: -2 shift left by 2
            xend = size(template_vessl, 2)+xoffset; %512-2, 510
            xbegin = 1; %fetches 1-510
        end
        if(xoffset >= 0) %for example: +3, shift right by 3, index starts 1
            xend = size(template_vessl, 2); %512
            xbegin = xoffset+1; %fetches 4-512
        end
        if(yoffset >= 0) %for example: -2 shift down by 2
            yend = size(template_vessl, 1); %256
            ybegin = yoffset+1; %fetches 2-254
        end
        if(yoffset < 0) %for example: +3, shift up by 3, index starts 1
            yend = size(template_vessl, 1) + yoffset; %256-3, 253
            ybegin = 1; %fetches 1-253
        end
        [k, ybegin, yend, xbegin, xend]
        recovered_vessl = zeros(sz);
        extracted_vessl = slice_vessl(ybegin:yend, xbegin:xend); %extract the overlap
        recovered_vessl(1:size(extracted_vessl,1), 1:size(extracted_vessl,2)) = extracted_vessl; %store registered
        nccVesslStk(k).vesselness = recovered_vessl;
        writeimg = uint8(255*(recovered_vessl./max(recovered_vessl(:))));
        imwrite(writeimg, outfname1 ,'tif', 'Compression', 'none', 'WriteMode', 'append');
    end
end

