% This file is part of "Motion Correction Microscopy" published as: 
% 
% Kumar et. al., "A Motion Correction Framework for Time Series 
% Sequences in Microscopy Images." Microscopy and Microanalysis. 2012. 
% 
% "Motion Correction Microscopy" is free software: you can redistribute it and/or modify
% it under the terms of the GNU Lesser Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% "Motion Correction Microscopy" is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Lesser Public License for more details.
% 
% You should have received a copy of the GNU Lesser Public License
% along with "Motion Correction Microscopy".  If not, see <http://www.gnu.org/licenses/>.
% 
% %%%%%%%%%%%%%%%%%%%%%%%% COPYRIGHT %%%%%%%%%%%%%%%%%%%%%%%%
% 
% Copyright (c) 2012, Kumar, Ankur N (ankur.kumar@vanderbilt.edu)
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without 
% modification, are permitted provided that the following conditions are 
% met:
% 
%     * Redistributions of source code must retain the above copyright 
%       notice, this list of conditions and the following disclaimer.
%     * Redistributions in binary form must reproduce the above copyright 
%       notice, this list of conditions and the following disclaimer in 
%       the documentation and/or other materials provided with the distribution
%     * Cite the Reference: 
%         
%         * Kumar et. al., "A Motion Correction Framework for Time Series 
%         Sequences in Microscopy Images." Microscopy and Microanalysis. 2012. 
%       
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
% AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
% IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
% ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
% LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
% CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
% SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
% CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
% ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
% POSSIBILITY OF SUCH DAMAGE.
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Please cite the publication:
% 
% Kumar et. al., "A Motion Correction Framework for Time Series 
% Sequences in Microscopy Images." Microscopy and Microanalysis. 2012. 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Author: Ankur Kumar
% PSF non paraxial
% lx ly lz = width(cols) height(rows) depth
%reference: Dusch, E., Dorval, T., Vincent, N., Wachsmuth, M. & Genovesio, A. (2007). 
%   Three-dimensional point spread function model for line-scanning confocal microscope with high-aperture objective. J Microsc 228, 132-138.


function [psf, h_ex, h_em] = calcPSFnp(lx,ly,lz, NA, n, s, lambda_ex, lambda_em)

alpha = asin(NA/n);
k_ex = n*2*pi/lambda_ex;
k_em = n*2*pi/lambda_em;
j=sqrt(-1);

h_ex = zeros(ly, lx, lz);
h_em = h_ex;
psf = h_ex;

syms B
syms St

for z=1:lz %depth
    for y=1:ly %height - row
        for x=1:lx %width - col
            %integrate from -1 to 1, paraxial
            h_ex_fun = @(B)exp(-j*k_ex*y*alpha*B)*exp(-j*0.5*k_ex*z*alpha.^2*B.^2)*k_ex;
            h_ex_val = double(int(h_ex_fun,B,-1,1));
            %integrate from 0 to 1, the following double integral is hard
            %to integrate.. is this even possible?
            % integrate bessel0 separately?
            h_em_fun = @(B)besselj(0,k_em*alpha*B*sqrt(x^2+(y-St)^2))*exp(-j*0.5*k_em*z*alpha.^2*B^2)*k_em*B;
            h_em_tmp = double(int(h_em_fun,B,0,1));
            h_em_tmp = abs(h_em_tmp).^2;
            h_em_val = double(int(h_em_tmp,St,-s,s));
            h_ex(y,x,z) = h_ex_val;
            h_em(y,x,z) = h_em_val;
            psf(y,x,z) = (abs(h_ex).^2).*h_em_val;
        end
    end
end

filepath = strcat(pwd,'/images/');
outfname = strcat(filepath, 'psf_l2m.tif');
if exist(outfname, 'file')
    delete(outfname);
end

for z=1:lz
    writeimg = uint8(psf(:,:,z));
    imwrite(writeimg, outfname ,'tif', 'Compression', 'none', 'WriteMode', 'append');
    z
end

