% This file is part of "Motion Correction Microscopy" published as: 
% 
% Kumar et. al., "A Motion Correction Framework for Time Series 
% Sequences in Microscopy Images." Microscopy and Microanalysis. 2012. 
% 
% "Motion Correction Microscopy" is free software: you can redistribute it and/or modify
% it under the terms of the GNU Lesser Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% "Motion Correction Microscopy" is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Lesser Public License for more details.
% 
% You should have received a copy of the GNU Lesser Public License
% along with "Motion Correction Microscopy".  If not, see <http://www.gnu.org/licenses/>.
% 
% %%%%%%%%%%%%%%%%%%%%%%%% COPYRIGHT %%%%%%%%%%%%%%%%%%%%%%%%
% 
% Copyright (c) 2012, Kumar, Ankur N (ankur.kumar@vanderbilt.edu)
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without 
% modification, are permitted provided that the following conditions are 
% met:
% 
%     * Redistributions of source code must retain the above copyright 
%       notice, this list of conditions and the following disclaimer.
%     * Redistributions in binary form must reproduce the above copyright 
%       notice, this list of conditions and the following disclaimer in 
%       the documentation and/or other materials provided with the distribution
%     * Cite the Reference: 
%         
%         * Kumar et. al., "A Motion Correction Framework for Time Series 
%         Sequences in Microscopy Images." Microscopy and Microanalysis. 2012. 
%       
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
% AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
% IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
% ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
% LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
% CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
% SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
% CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
% ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
% POSSIBILITY OF SUCH DAMAGE.
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Please cite the publication:
% 
% Kumar et. al., "A Motion Correction Framework for Time Series 
% Sequences in Microscopy Images." Microscopy and Microanalysis. 2012. 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Author: Ankur Kumar
% PSF paraxial
% lx ly lz = width(cols) height(rows) depth
%reference: Wolleschensky, R., Zimmermann, B. & Kempe, M. (2006). 
%    High-speed confocal fluorescence imaging with a novel line scanning microscope. J Biomed Optic 11, 064011.  
%reference: Sandison, D.R. & Webb, W.W. (1994). 
%   Background rejection and signal-to-noise optimization in confocal and alternative fluorescence microscopes. Appl Optic 33, 603-615.

function [psf, h_ex, h_em] = calcPSFp(lx,ly,lz, NA, n, s, lambda_ex, lambda_em)

alpha = asin(NA/n);
k_ex = n*2*pi/lambda_ex;
k_em = n*2*pi/lambda_em;
j=sqrt(-1);

h_ex = zeros(ly, lx, lz);
h_em = h_ex;
psf = h_ex;

%Quadrature Gauss-Kronod, from Mathworks
%The quadgk function may be most efficient for high accuracies and oscillatory integrands. 
%It supports infinite intervals and can handle moderate singularities at the endpoints. 
%It also supports contour integration along piecewise linear paths.


%equally spaced points for Boole EM
intervals = linspace(-s,s,5);
h = intervals(2)-intervals(1);
h_em_vec = zeros(length(intervals));

intervals_em = linspace(0,1,5);
hemint = intervals_em(2)-intervals_em(1);
h_em_vec_in = zeros(length(intervals_em));

%Boole Ex
intervals_ex = linspace(-1,1,5);
hexint = intervals_ex(2)-intervals_ex(1);
h_ex_vec = zeros(length(intervals_ex));

% lz=31;
% cz=31;

%lzu = 2

lzu = floor(lz/2);
lyu = floor(ly/2);
lxu = floor(lx/2);
offsetz = mod(lz,2) + lzu;
offsety = mod(ly,2) + lyu;
offsetx = mod(lx,2) + lxu;


for z=-lzu+1:lzu %depth, EVEN
    %change x y z, make center of stack as 0, where is 0 in zstack
    z+offsetz
    tic
    for y=-lyu+1:lyu %height - row, EVEN
        for x=-lxu+1:lxu %width - col, EVEN
            %[x y]
            %integrate from -1 to 1, paraxial
            %excitation part - IS THERE A NEGATIVE SIGN for j*0.5???
            h_ex_fun = @(t)exp(-j.*k_ex.*y.*alpha.*t).*exp(j.*0.5.*k_ex.*z.*(alpha.^2).*(t.^2)).*k_ex;
%             [h_ex_val,err_h_ex_val] = quadgk(h_ex_fun,-1,1); %numerical
%             integration
            %use boole
            for indx = 1:length(intervals_ex)
                ts = intervals_ex(indx);
                h_ex_vec(indx) = h_ex_fun(ts);
            end
            
            h_ex_tmp = (2*hexint/45)*(7*h_ex_vec(1)+32*h_ex_vec(2)+12*h_ex_vec(3)+32*h_ex_vec(4)+7*h_ex_vec(5));

            h_ex_val = abs(h_ex_tmp).^2;
            %integrate from -s to s, 0 to 1, paraxial
            %emission part - IS THERE A x.^2 for the inner integral??
            %IS THERE A NEGATIVE SIGN for j*0.5???
            
            for indx = 1:length(intervals)
                ys = intervals(indx);
                h_em_fun = @(t)besselj(0,k_em.*alpha.*t.*sqrt(x.^2+(y-ys).^2)).*exp(j.*0.5.*k_em.*z.*(alpha.^2).*(t.^2)).*k_em.*t;
                for indx2 = 1:length(intervals_em)
                    es = intervals_em(indx2);
                    h_em_vec_in(indx2) = h_em_fun(es);
                end
                %h_em_tmp = quadgk(h_em_fun,0,1);
                h_em_tmp = (2*hemint/45)*(7*h_em_vec_in(1)+32*h_em_vec_in(2)+12*h_em_vec_in(3)+32*h_em_vec_in(4)+7*h_em_vec_in(5));
                h_em_vec(indx) = abs(h_em_tmp).^2; %f(x)
            end
            
            %boole value
            h_em_val = (2*h/45)*(7*h_em_vec(1)+32*h_em_vec(2)+12*h_em_vec(3)+32*h_em_vec(4)+7*h_em_vec(5));
            
            iz = offsetz + z;
            iy = offsety + y;
            ix = offsetx + x;
            %[iz z offsetz]
            %[ix iy iz]
            h_ex(iy,ix,iz) = h_ex_val;
            h_em(iy,ix,iz) = h_em_val;
            psf(iy,ix,iz) = h_ex(iy,ix,iz).*h_em(iy,ix,iz);
        end
    end
    toc
end


