% This file is part of "Motion Correction Microscopy" published as: 
% 
% Kumar et. al., "A Motion Correction Framework for Time Series 
% Sequences in Microscopy Images." Microscopy and Microanalysis. 2012. 
% 
% "Motion Correction Microscopy" is free software: you can redistribute it and/or modify
% it under the terms of the GNU Lesser Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% "Motion Correction Microscopy" is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Lesser Public License for more details.
% 
% You should have received a copy of the GNU Lesser Public License
% along with "Motion Correction Microscopy".  If not, see <http://www.gnu.org/licenses/>.
% 
% %%%%%%%%%%%%%%%%%%%%%%%% COPYRIGHT %%%%%%%%%%%%%%%%%%%%%%%%
% 
% Copyright (c) 2012, Kumar, Ankur N (ankur.kumar@vanderbilt.edu)
% All rights reserved.
% 
% Redistribution and use in source and binary forms, with or without 
% modification, are permitted provided that the following conditions are 
% met:
% 
%     * Redistributions of source code must retain the above copyright 
%       notice, this list of conditions and the following disclaimer.
%     * Redistributions in binary form must reproduce the above copyright 
%       notice, this list of conditions and the following disclaimer in 
%       the documentation and/or other materials provided with the distribution
%     * Cite the Reference: 
%         
%         * Kumar et. al., "A Motion Correction Framework for Time Series 
%         Sequences in Microscopy Images." Microscopy and Microanalysis. 2012. 
%       
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
% AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
% IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
% ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
% LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
% CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
% SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
% CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
% ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
% POSSIBILITY OF SUCH DAMAGE.
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Please cite the publication:
% 
% Kumar et. al., "A Motion Correction Framework for Time Series 
% Sequences in Microscopy Images." Microscopy and Microanalysis. 2012. 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%File: vesselnessDriver.m
%Purpose: compute vesselness, and run motion-correction

%Parameters -- Please look at the example .PBS file
%Note: STK_FIRST_TIME = 0 for motion-correction to run (Template file
    %already exists), STK_FIRST_TIME = 1, creates template file and does not
    %run motion-correction), movements: [less, moderate, most]
%filepath - path to deconvolved images
%filepath_orig - path to channel separated (not deconvolved) images (original)
%filename1 - filename of the image to correct, example: '0004_4t_ch2_1-750_sm_deconv_15CLAHE'
%filename_vessl_orig - filename header for vessel stacks, example: '0004_4t_ch2_1-750'
%filename_rbc_orig - filename header for vessel stacks, example: '0004_4t_ch1_1-750'
%filepath_vessl - path to results
%TemplateMATFilename - filename of template file .mat (Matlab) for the image
    %TemplateMATFilename = string concatenation of (filename1,'_TEMPLATE_maxVessl.mat')
%STK_FIRST_TIME - create template file, Does not run motion-correction
%low_range, high_range - range of frame numbers used to compute template, used when STK_FIRST_TIME = 1
%epsillon_rigid - threshold for rigid registration [15, 25, 45]
%epsillon_nonrigid - threshold for nonrigid registration [4, 4, 8]
%number_samples - number of samples for registration
%lambda_in - regularization parameter, 10^5 - 10^6
%r - annealing parameter, 0.93

function maxVesslStk = vesselnessDriver(filepath,filepath_orig,filename1,filename_vessl_orig, filename_rbc_orig, filepath_vessl,TemplateMATFilename,STK_FIRST_TIME,low_range, high_range,epsillon_rigid,epsillon_nonrigid,number_samples,lambda_in,r)

%compute time taken
STARTTIME = cputime;

fname1 = strcat(filepath, filename1,'.tif'); %file extension
fname2 = strcat(filepath_orig, filename_vessl_orig,'.tif'); %file extension
fname3 = strcat(filepath_orig, filename_rbc_orig,'.tif'); %file extension

stk_vessl = tiffread29(fname1) %read the stk
lz = length(stk_vessl);
lx = stk_vessl(1,1).width;
ly = stk_vessl(1,1).height;

%vesselness Tubular structure detector
sigvec_vessl = [1 2 3 4 5 6]; %ranges for vessel detector
delta_vessl = 15; 
gamma = 1; 
beta = .5;
eigHstk = computeFeatureValuesEig(stk_vessl, sigvec_vessl,delta_vessl,gamma,beta);
tic
maxVesslStk = computeMaxVesselness(eigHstk);
toc

%find frequencies of sigma in maxVessel strength
freq_sig_vessl = zeros(size(sigvec_vessl));
ss = maxVesslStk(1,2).maxSigma; %choose some random slice
for kcx = 1:length(sigvec_vessl)
    freq_sig_vessl(kcx) = length(find(ss==sigvec_vessl(kcx)));
end

savefilename = strcat(filename1,'_maxVessl');
%save(savefilename,'maxVesslStk','freq_sig_vessl','-v7.3');

%%%%%%%%%%%%%%%%%WRITE FILES%%%%%%%%%%%%%%%%%%%%%%
WRITE_FLAG = 1;
filepath1 = filepath_vessl; %CHANGE TO OUTPUT PATH
if(WRITE_FLAG)
    
    outfn1 = strcat(filepath1, filename1);
    %%WRITE IMG AT MAX VESSEL LEVEL
    writesuperimg = zeros(ly,lx);
    writeimgsum = writesuperimg;
   
    %vessel files
    outfname1 = strcat(outfn1, '_sigMAX','_del',num2str(delta_vessl),'_vessl.tif');
    outfname5 = strcat(outfn1, '_2XsigMAX','_del',num2str(delta_vessl),'_vessl.tif');
    outfname6 = strcat(outfn1, '_2XsigMAX','_del',num2str(delta_vessl),'_vessl_AVERAGE.tif');

    for k=1:lz
        img = maxVesslStk(k).vesselness;
        writeimg = img;
        imwrite(writeimg, outfname1 ,'tif', 'Compression', 'none', 'WriteMode', 'append');
        writeimg = 2.*img; %2X MAX vesselness
        imwrite(writeimg, outfname5 ,'tif', 'Compression', 'none', 'WriteMode', 'append');
        writeimgsum = writeimgsum + writeimg; %create sum of vesselness
    end
    writeimgsum = writeimgsum./lz; %create average 2X MAX Vesselness
    imwrite(writeimgsum, outfname6 ,'tif', 'Compression', 'none', 'WriteMode', 'append');
end

%%RUN REST OF PROGRAMS
disp('TOTAL TIME for computing Vesselness')
TOTALTIME1 = cputime-STARTTIME

stk_vessl = tiffread29(fname1) %read the stk
stk_vessl_orig = tiffread29(fname2) %read the stk
stk_rbc_orig = tiffread29(fname3) %read the stk
disp('Executing Motion Correction or Finding Template maxVesslStk..')
maxVesslStk = driver_main2(filepath1,filename1,TemplateMATFilename,maxVesslStk,stk_vessl,stk_vessl_orig,stk_rbc_orig,STK_FIRST_TIME,low_range, high_range,epsillon_rigid,epsillon_nonrigid,number_samples,lambda_in,r);


